searchData={"content_type":"text/markdown","items":[{"doc":"Module to manage reactive state by using GenServer processes (\"reactive process\" from here on) to manage each piece of state and its relationships to other reactive processes.","ref":"Reactive.html","title":"Reactive","type":"module"},{"doc":"The package can be installed by adding `reactive_state` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:reactive_state, \"~> 0.2.3\"}\n  ]\nend\n```","ref":"Reactive.html#module-installation","title":"Installation - Reactive","type":"module"},{"doc":"To automatically import the `reactive/1` and `reactive/2` macros, you can use `use Reactive` which is the equivalent of:\n\n```elixir\nimport Reactive, only: [reactive: 1, reactive: 2]\nalias Reactive.Ref\n```\n\nExample usage:\n\n    iex> use Reactive\n    iex> ref = reactive(do: 2)\n    iex> ref_squared = reactive do\n    ...>   get(ref) ** 2\n    ...> end\n    iex> Reactive.get(ref_squared)\n    4\n    iex> Ref.set(ref, 3)\n    iex> Reactive.get(ref_squared)\n    9\n\nTo set options at the module level, you can pass options, for example:\n\n    iex> defmodule ReactiveExample do\n    ...>   use Reactive, reactive: :reactive_protected, ref: :ref_protected, opts: [gc: false]\n    ...>\n    ...>   def run do\n    ...>     value = ref_protected(0)\n    ...>     computed = reactive_protected do\n    ...>       get(value) + 1\n    ...>     end\n    ...>     {Ref.get(value), Ref.get(computed)}\n    ...>   end\n    ...> end\n    iex>\n    iex> ReactiveExample.run()\n    {0, 1}","ref":"Reactive.html#module-reactive-block","title":"Reactive Block - Reactive","type":"module"},{"doc":"iex> alias Reactive.Ref\n    iex> ref = Ref.new(0) #PID \n    iex> Ref.get(ref) # or Ref.get(ref)\n    0\n    iex> Ref.set(ref, 1)\n    :ok\n    iex> Ref.get(ref)\n    1","ref":"Reactive.html#module-working-with-data-directly-with-reactive-ref","title":"Working with data directly with `Reactive.Ref` - Reactive","type":"module"},{"doc":"By default, new reactive processes will be linked to the current process.\nTo override this behavior, pass the `supervisor` keyword arg with the name of your DynamicSupervisor during process creation:\n\n    value = Ref.new(0, supervisor: MyApp.Supervisor)\n    computed = reactive supervisor: MyApp.Supervisor do\n      get(value) + 1\n    end\n\nYou can also pass default options like this:\n\n    use Reactive, ref: :ref, opts: [supervisor: MyApp.Supervisor]\n    ...\n    value = ref(0)\n    computed = reactive do\n      get(value) + 1\n    end","ref":"Reactive.html#module-supervisor","title":"Supervisor - Reactive","type":"module"},{"doc":"If a reactive process has been killed for any reason, it will be restarted upon a `Reactive.get` or `Ref.get` call:\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = Ref.new(0)\n    iex> DynamicSupervisor.terminate_child(Reactive.Supervisor, ref)\n    iex> Ref.get(ref)\n    0","ref":"Reactive.html#module-process-restarting","title":"Process Restarting - Reactive","type":"module"},{"doc":"The default garbage collection strategy is to kill any processes that were not accessed through\na `Reactive.get` or `Ref.get` call between GC calls:\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = Ref.new(0)\n    iex> Reactive.Supervisor.gc()\n    iex> nil == Reactive.resolve_process(ref)\n\nReactive processes can be protected with the `gc` option:\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = Ref.new(0, gc: false)\n    iex> Reactive.Supervisor.gc()\n    iex> ^ref = Reactive.resolve_process(ref)\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = reactive gc: false do\n    ...>    # some expensive computation\n    ...> end\n    iex> Reactive.Supervisor.gc()\n    iex> ^ref = Reactive.resolve_process(ref)","ref":"Reactive.html#module-garbage-collection","title":"Garbage Collection - Reactive","type":"module"},{"doc":"You can name a reactive process using the `name` option:\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> Ref.new(0, name: MyApp.Value)\n    iex> reactive name: MyApp.Computed do\n    ...>   get(MyApp.Value) + 1\n    ...> end\n    iex> Ref.get(MyApp.Value)\n    0\n    iex> Reactive.get(MyApp.Computed)\n    1","ref":"Reactive.html#module-named-process","title":"Named Process - Reactive","type":"module"},{"doc":"Proactive reactive processes will not trigger immediately after a dependency changes; they must triggered with a call to `Reactive.Supervisor.trigger_proactive`\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> num = Ref.new(0)\n    iex> ref =\n    ...>   reactive proactive: true do\n    ...>     get(num) + 1\n    ...>   end\n    iex> Reactive.get_cached(ref)\n    1\n    iex> Ref.set(num, 1)\n    iex> Reactive.Supervisor.trigger_proactive()\n    iex> Reactive.get_cached(ref)\n    2","ref":"Reactive.html#module-proactive-process","title":"Proactive Process - Reactive","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Reactive.html#child_spec/1","title":"Reactive.child_spec/1","type":"function"},{"doc":"Retrieve the state of a reactive process","ref":"Reactive.html#get/2","title":"Reactive.get/2","type":"function"},{"doc":"iex> ref = Reactive.new(fn _ -> 0 end)\n    iex> Reactive.get(ref)\n    0","ref":"Reactive.html#get/2-example","title":"Example - Reactive.get/2","type":"function"},{"doc":"Retrieve the cached state of a reactive process, or :stale if it has not been computed or is stale","ref":"Reactive.html#get_cached/1","title":"Reactive.get_cached/1","type":"function"},{"doc":"iex> use Reactive\n    iex> ref = reactive do\n    ...>   0\n    ...> end\n    iex> Reactive.get_cached(ref)\n    :stale\n    iex> Reactive.get(ref)\n    0\n    iex> Reactive.get_cached(ref)\n    0","ref":"Reactive.html#get_cached/1-example","title":"Example - Reactive.get_cached/1","type":"function"},{"doc":"Create a reactive process using a method\n\n    iex> use Reactive\n    iex> ref = Ref.new(2)\n    iex> ref_squared = Reactive.new(fn call_id ->\n    ...>   Reactive.get(ref, call_id: call_id) ** 2\n    ...> end)\n    iex> Reactive.get(ref_squared)\n    4","ref":"Reactive.html#new/2","title":"Reactive.new/2","type":"function"},{"doc":"Syntatic sugar for creating reactive blocks\n\n    iex> use Reactive\n    iex> ref = Ref.new(2)\n    iex> ref_squared = reactive do\n    ...>   get(ref) ** 2\n    ...> end #PID \n    iex> Reactive.get(ref_squared)\n    4\n    iex> Ref.set(ref, 3)\n    iex> Reactive.get(ref_squared)\n    9\n\nUsing the `reactive` macro in this way is roughly equivalent to:\n\n    iex> use Reactive\n    iex> ref = Ref.new(2)\n    iex> ref_squared = Reactive.new(fn from ->\n    ...>   Reactive.get(ref, from: from) ** 2\n    ...> end)\n    iex> Reactive.get(ref_squared)\n    4\n    iex> Ref.set(ref, 3)\n    iex> Reactive.get(ref_squared)\n    9","ref":"Reactive.html#reactive/1","title":"Reactive.reactive/1","type":"macro"},{"doc":"Create a reactive process with options:\n\n```elixir\nreactive name: MyApp.SomeValue, gc: false, proactive: true, supervisor: MyApp.DynamicSupervisor do\n  # ...\nend\n```","ref":"Reactive.html#reactive/2","title":"Reactive.reactive/2","type":"macro"},{"doc":"Find a reactive process from a pid or alias.\n\n    iex> use Reactive\n    iex> pid = Ref.new(0, name: MyApp.Value)\n    iex> true = Reactive.resolve_process(MyApp.Value) == Reactive.resolve_process(pid)\n\nYou can ensure a process will be returned by passing the `create: true` option\n\n```elixir\nReactive.resolve_process(pid, create: true)\n```","ref":"Reactive.html#resolve_process/2","title":"Reactive.resolve_process/2","type":"function"},{"doc":"Replace a reactive process's computation method\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = reactive do\n    ...>   0\n    ...> end\n    iex> Reactive.get(ref)\n    0\n    iex> Reactive.set(ref, fn _ -> 1 end)\n    :ok\n    iex> Reactive.get(ref)\n    1","ref":"Reactive.html#set/3","title":"Reactive.set/3","type":"function"},{"doc":"Wrapper to work with reactive state directly","ref":"Reactive.Ref.html","title":"Reactive.Ref","type":"module"},{"doc":"iex> use Reactive\n    iex> ref = Ref.new(0) #PID \n    iex> Ref.get(ref)\n    0\n    iex> Ref.set(ref, 1)\n    :ok\n    iex> Ref.get(ref)\n    1","ref":"Reactive.Ref.html#module-example","title":"Example - Reactive.Ref","type":"module"},{"doc":"Retrieve the state of a reactive process","ref":"Reactive.Ref.html#get/2","title":"Reactive.Ref.get/2","type":"function"},{"doc":"iex> use Reactive\n    iex> ref = Ref.new(0)\n    iex> Ref.get(ref)\n    0","ref":"Reactive.Ref.html#get/2-example","title":"Example - Reactive.Ref.get/2","type":"function"},{"doc":"Create a reactive process","ref":"Reactive.Ref.html#new/2","title":"Reactive.Ref.new/2","type":"function"},{"doc":"iex> use Reactive\n    iex> ref = Ref.new(0)\n    iex> Ref.get(ref)\n    0","ref":"Reactive.Ref.html#new/2-example","title":"Example - Reactive.Ref.new/2","type":"function"},{"doc":"Set the state for a reactive process","ref":"Reactive.Ref.html#set/2","title":"Reactive.Ref.set/2","type":"function"},{"doc":"iex> use Reactive\n    iex> ref = Ref.new(0)\n    iex> Ref.set(ref, 1)\n    :ok\n    iex> Ref.get(ref)\n    1","ref":"Reactive.Ref.html#set/2-example","title":"Example - Reactive.Ref.set/2","type":"function"},{"doc":"Set the state for a reactive process through a function that receives the old value","ref":"Reactive.Ref.html#set_fn/2","title":"Reactive.Ref.set_fn/2","type":"function"},{"doc":"iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = Ref.new(0)\n    iex> Ref.set_fn(ref, fn state -> state + 1 end)\n    :ok\n    iex> Ref.get(ref)\n    1","ref":"Reactive.Ref.html#set_fn/2-example","title":"Example - Reactive.Ref.set_fn/2","type":"function"},{"doc":"Default DynamicSupervisor to manage task creation and garbage collection.","ref":"Reactive.Supervisor.html","title":"Reactive.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Reactive.Supervisor.html#child_spec/1","title":"Reactive.Supervisor.child_spec/1","type":"function"},{"doc":"Start the supervisor if needed (not recommended)","ref":"Reactive.Supervisor.html#ensure_started/0","title":"Reactive.Supervisor.ensure_started/0","type":"function"},{"doc":"Garbage collect processes for a supervisor (Reactive.Supervisor by default)\n\nOptions:\n- name: [pid or alias] (default: `Reactive.Supervisor`)\n- strategy: :counter (default `:counter`)\n- count: [integer] (default `nil`)\n- random: [boolean] (default `false`)\n\nExample:\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = Ref.new(0)\n    iex> Reactive.Supervisor.gc()\n    iex> nil == Reactive.resolve_process(ref)\n\nReactive processes can be protected with the `gc` option:\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = Ref.new(0, gc: false)\n    iex> Reactive.Supervisor.gc()\n    iex> ^ref = Reactive.resolve_process(ref)\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> ref = reactive gc: false do\n    ...>    # some expensive computation\n    ...> end\n    iex> Reactive.Supervisor.gc()\n    iex> ^ref = Reactive.resolve_process(ref)","ref":"Reactive.Supervisor.html#gc/1","title":"Reactive.Supervisor.gc/1","type":"function"},{"doc":"","ref":"Reactive.Supervisor.html#start_link/1","title":"Reactive.Supervisor.start_link/1","type":"function"},{"doc":"Trigger computations for any stale reactive processes who have the option `proactive: true`\n\nExample:\n\n    iex> use Reactive\n    iex> Reactive.Supervisor.ensure_started()\n    iex> num = Ref.new(0)\n    iex> ref =\n    ...>   reactive proactive: true do\n    ...>     get(num) + 1\n    ...>   end\n    iex> Reactive.get_cached(ref)\n    1\n    iex> Ref.set(num, 1)\n    iex> Reactive.Supervisor.trigger_proactive()\n    iex> Reactive.get_cached(ref)\n    2","ref":"Reactive.Supervisor.html#trigger_proactive/1","title":"Reactive.Supervisor.trigger_proactive/1","type":"function"}]}